! vim:fdm=marker:
!------------------------------------------------------------------------------------------
!
!                                create_V_ext
!
!------------------------------------------------------------------------------------------


PROGRAM create_V_ext
!
!       Purpose: writes a file V_ext<tag>.inp  containing an 
!                external one-body potential that can be read in  by openMCTDHB.
!                reads the file V_ext.nml. Add whatever potential you like.
!       
!       Record of revisions:
!               Date            Programmer              Description of change
!               ====            ==========              ===================== 
!               16.02.2011      K. Sakmann              Original code
!
        
USE   moduleparameters
USE   moduleinputvariables
USE   moduleallocatables
USE   modulecreate_V_ext
USE   modulegrid

IMPLICIT NONE
CHARACTER(255)       :: filename
CHARACTER(70)        :: potName
REAL*8  :: x0                   =       0.d0
REAL*8  :: y0                   =       0.d0
REAL*8  :: z0                   =       0.d0
REAL*8  :: w_x                  =       0.d0
REAL*8  :: w_y                  =       0.d0
REAL*8  :: w_z                  =       0.d0



!------------- READ INPUT PARAMETERS --------------------------------- 

NAMELIST /GeneralPars/ xi,xf,yi,yf,zi,zf,NDX,NDY,NDZ,Dims
NAMELIST /PotentialName/ potName
NAMELIST /TagName/ tag
NAMELIST /HarmonicParameters/ x0,y0,z0,w_x,w_y,w_z  


open(110,FILE='V_ext.nml',DELIM='APOSTROPHE')
  read(110,NML=GeneralPars)
  read(110,NML=TagName)
  read(110,NML=PotentialName)
close(110)

!------------- CONSISTENCY CHECKS OF THE INPUT PARAMETERS -----------------

call check_grid(Dims,NDX,NDY,NDZ)  ! CHECK THE GRID AND DIMENSIONS

!------------- INITIALIZE VARIABLES THAT DEPEND ON THE INPUT VARIABLES

tag = TRIM(ADJUSTL(tag))
filename = trim('V_ext'//trim(tag)//'.inp')


!------------- ALLOCATE AND (INITIALIZE TO ZERO) ALL SYSTEM SIZE DEPENDENT ARRAYS --------------------
!              ADD MORE ALLOCATABLE ARRAYS TO MODULEALLOCATABLES 

CALL allocate_systemArrays(NDX,NDY,NDZ,Morb=1,nConf=1,nSuperDiags=0)
CALL get_positionGrid(xi,xf,NDX,gridx)
CALL get_positionGrid(yi,yf,NDY,gridy)
CALL get_positionGrid(zi,zf,NDZ,gridz)

!------------ CHOOSE THE EXTERNAL POTENTIAL HERE DEPENDING ON THE INPUT
IF (TRIM(ADJUSTL(potName)) =='HARMONIC') THEN 

  open(110,FILE='V_ext.nml',DELIM='APOSTROPHE')
    read(110,NML=HarmonicParameters)
  close(110)

  WRITE(*,*)"USE A HARMONIC POTENTIAL"
  WRITE(*,*)"x0=",x0,"y0=",y0,"z0",z0
  CALL get_harmonicPotential(NDX,NDY,NDZ,gridx,gridy,gridz,x0,y0,z0,&
                             w_x,w_y,w_z,V_ext) 

  WRITE(*,*)"SAVE THE INPUT FILE IN  ",'V_ext'//trim(adjustl(tag))//'.nml'
  OPEN(113,FILE='V_ext'//trim(adjustl(tag))//'.nml',DELIM='APOSTROPHE',status='REPLACE')
    WRITE(113,NML=GeneralPars)
    WRITE(113,NML=TagName)
    WRITE(113,NML=PotentialName)
    WRITE(113,NML=HarmonicParameters)
  CLOSE(113)

ELSE IF (TRIM(ADJUSTL(potName)) == 'BJJ')  THEN

  WRITE(*,*)"USE THE BJJ-LETTER POTENTIAL"
  CALL get_bjjLetterPotential(NDX,NDY,NDZ,gridx,gridy,gridz,Dims,V_ext) 

  open(113,FILE='V_ext'//trim(adjustl(tag))//'.nml',DELIM='APOSTROPHE',status='REPLACE')
    write(113,NML=GeneralPars)
    write(113,NML=TagName)
    write(113,NML=PotentialName)
  close(113)

ELSE 

  WRITE(*,*)"ERROR WRITE UNKNOWN POTENTIAL", potName
  STOP

END IF

WRITE(*,*)"WRITE THE  POTENTIAL TO ",filename
CALL write_potentialToFile(NDX,NDY,NDZ,gridx,gridy,gridz,V_ext,filename)

CALL deallocate_systemArrays

END PROGRAM create_V_ext
