! vim:fdm=marker:
!-----------------------------------------------------------------------------
!> This module contains routines that deal with I/O to the screen and to files
!>
!>\authors Kaspar Sakmann (2007)
!>\since (2007)
!>\todo many routines still need variable documentation
MODULE moduleinputoutput
IMPLICIT NONE


CONTAINS

!------------------------------------------------------------------------------
!> This routine prints the OpenMCTDHB header to the screen
!>
!> \author Kaspar Sakmann (2010)
!> \since  2010
SUBROUTINE print_header
!---{{{
IMPLICIT NONE
WRITE(*,*)
WRITE(*,1100)" ##############################################################################################################"
WRITE(*,1100)" #                        ___                   __  __  ____ _____ ____  _   _ ____                           #"
WRITE(*,1100)" #                       / _ ` _ __   ___ _ __ |  `/  |/ ___|_   _|  _ `| | | | __ )                          #"
WRITE(*,1100)" #                      | | | | '_ ` / _ ` '_ `| |`/| | |     | | | | | | |_| |  _ `                          #"
WRITE(*,1100)" #                      | |_| | |_) |  __/ | | | |  | | |___  | | | |_| |  _  | |_) |                         #"
WRITE(*,1100)" #                       `___/| .__/ `___|_| |_|_|  |_|`____| |_| |____/|_| |_|____/                          #"
WRITE(*,1100)" #                            |_|                                                                             #" 
WRITE(*,1100)" #                                                                                                            #" 
WRITE(*,1100)" #                                        (2007-2012)   Version 2.3                                           #"
WRITE(*,1100)" #                                                                                                            #" 
WRITE(*,1100)" #       Kaspar Sakmann (head of project), Axel Lode, Alexej Streltsov, Ofir Alon, Lorenz Cederbaum           #" 
WRITE(*,1100)" #                                                                                                            #" 
WRITE(*,1100)" ##############################################################################################################"
WRITE(*,*)

1100 FORMAT(116A)
!---}}}
END SUBROUTINE print_header

!-------------------------------------------------------------------------------
!> This routine prints info about the current run to the screen and performs 
!> a few basic consistency checks 
!> 
!> \authors Kaspar Sakmann (2007)
!> \since  2007
SUBROUTINE print_runInfo(lambda0,NPar,MOrb,Dims,NDX,NDY,NDZ,xi,xf,yi,yf,zi,zf,tinyStep,printStep,printAllStep, &
              printCorrelationFuncs,AbsTime,maxTime,TolError,restart,Relax,readPotential,propDirection,tag)
!---{{{
IMPLICIT NONE
REAL*8,INTENT(IN)  :: lambda0              
INTEGER ,INTENT(IN):: NPar                 
INTEGER,INTENT(IN) :: MOrb                 
INTEGER,INTENT(IN) :: Dims                

INTEGER,INTENT(IN) :: NDX                  
INTEGER,INTENT(IN) :: NDY                  
INTEGER,INTENT(IN) :: NDZ                  
REAL*8,INTENT(IN)  :: xi                   
REAL*8,INTENT(IN)  :: xf                   
REAL*8,INTENT(IN)  :: yi                   
REAL*8,INTENT(IN)  :: yf                   
REAL*8,INTENT(IN)  :: zi                   
REAL*8,INTENT(IN)  :: zf                   

REAL*8,INTENT(IN)  :: tinyStep             
REAL*8,INTENT(IN)  :: printStep            
REAL*8,INTENT(IN)  :: printAllStep         
INTEGER,INTENT(IN) :: printCorrelationFuncs      
                             
REAL*8,INTENT(IN)  :: AbsTime             
REAL*8,INTENT(IN)  :: maxTime              
REAL*8,INTENT(IN)  :: TolError             
LOGICAL,INTENT(IN) :: restart              
LOGICAL,INTENT(IN) :: Relax                
LOGICAL,INTENT(IN) :: readPotential        
INTEGER,INTENT(IN) :: propDirection        
CHARACTER(55),INTENT(IN)  :: tag           

WRITE(*,*)
IF (Relax) THEN 

  IF (propDirection == 1) THEN
    WRITE(*,109)"Relaxation run. Propagate in imaginary time to the ground state."
  ELSE IF (propDirection == -1) THEN 
    WRITE(*,112)"Relax = ",Relax," propDirection",propDirection
    WRITE(*,109)"RELAXATION IN THE WRONG DIRECTION. THIS WILL BLOW UP. I WILL THEREFORE STOP."
    WRITE(*,*)
    STOP
  ELSE 
    WRITE(*,110)"ILLEGAL VALUE FOR propagationDirection",propDirection
    STOP
  END IF

ELSE 
  IF (propDirection == 1) THEN
    WRITE(*,109)"Propagation run. Propagate forward in time."
  ELSE IF (propDirection == -1) THEN 
    WRITE(*,109)"Propagation run. Propagate backwards in time (count forwards)."
  ELSE
    WRITE(*,110)"ILLEGAL VALUE FOR propagationDirection: ",propDirection
    STOP
  END IF

END IF

IF (MOrb== 1) THEN
  WRITE(*,109)"Gross-Pitaevskii computation"
ELSE IF (MOrb == 2) THEN 
  WRITE(*,109)"Many-body computation"
ELSE IF (MOrb > 2) THEN 
  WRITE(*,109)"More than two orbitals will be available in the next release, but not yet."
  STOP
ELSE 
  WRITE(*,110)"ILLEGAL VALUE FOR MOrb",MOrb
  STOP
END IF

IF (restart) THEN
  WRITE(*,109)"Restart from restart.orbs.inp  and restart.coef.inp "
ELSE IF (.NOT. restart) THEN 
  WRITE(*,109)"Start from default orbitals: harmonic oscillator eigenfunctions"
ELSE  
  WRITE(*,*)"ILLEGAL VALUE FOR restart",restart
  STOP
END IF

IF (readPotential) THEN
  WRITE(*,109)"external potential will be read from V_ext.inp "
ELSE IF (.NOT. readPotential) THEN 
  WRITE(*,109)"external potential defined in get_V_ext"
ELSE  
  WRITE(*,*)"ILLEGAL VALUE FOR readPotential",readPotential
  STOP
END IF




WRITE(*,*)

IF (Dims < 4 .AND. Dims >0) THEN
        WRITE(*,113)"Number of dimensions               = ",Dims
ELSE 
  WRITE(*,113)"INVALID NUMBER OF DIMENSIONS ",Dims
  STOP
END IF
WRITE(*,113)"Number of bosons                   = ",NPar
WRITE(*,113)"Number of orbitals                 = ",MOrb
WRITE(*,111)"Interaction strength lambda0       = ",lambda0
WRITE(*,111)"lambda0(N-1)                       = ",lambda0*(NPar-1)

WRITE(*,*)

WRITE(*,117)"x box: [",xi,",",xf,")     grid points: ",NDX
WRITE(*,117)"y box: [",yi,",",yf,")     grid points: ",NDY
WRITE(*,117)"z box: [",zi,",",zf,")     grid points: ",NDZ
WRITE(*,*)
WRITE(*,114)"Initial Time                       = ",AbsTime
WRITE(*,114)"Final Time                         = ",maxTime
WRITE(*,*)
WRITE(*,115)"Error tolerance                    = ",TolError
WRITE(*,*)

109 FORMAT(5X,A)
110 FORMAT(5X,A60,I6)
111 FORMAT(5X,A45,F13.9)
112 FORMAT(5X,A15,L,A15,I6)
113 FORMAT(5X,A45,I6)
114 FORMAT(5X,A45,F19.9)
115 FORMAT(5X,A45,ES14.4)
117 FORMAT(13X,A,F8.2,A,F8.2,A,I8)
!---}}}
END SUBROUTINE print_runInfo

!-----------------------------------------------------------------------------
!> This routine writes a double value in ten digit format,  useful 
!> for naming files without blanks       
!> 
!> \authors Kaspar Sakmann (2007)
!> \since (2007)
SUBROUTINE get_doubleAsString(x,doubleString)
!       {{{ 
IMPLICIT NONE
INTEGER,parameter                    :: DBL=8
real(kind=DBL),INTENT(IN)            :: x
CHARACTER(len=10),INTENT(OUT)        :: doubleString

IF (x <= 999999.999d0) WRITE(doubleString,'(F10.3)') x
IF (x <= 99999.9999d0) WRITE(doubleString,'(F10.4)') x
IF (x <= 9999.99999d0) WRITE(doubleString,'(F10.5)') x
IF (x <= 999.999999d0) WRITE(doubleString,'(F10.6)') x
IF (x <= 99.9999999d0) WRITE(doubleString,'(F10.7)') x
IF (x <= 9.99999999d0) WRITE(doubleString,'(F10.8)') x

IF ((x < 0.d0) .OR. (x >= 1000000.0d0)) THEN
  WRITE(6,*)"x not implemented"
  STOP
END IF

!}}}
END SUBROUTINE get_doubleAsString

!----------------------------------------------------------------------------
!> This routine creates a character string 'identifier' that
!> is used to identify a given run in all outputfiles
!> 
!> \authors Kaspar Sakmann (2007)
!> \since (2007)
SUBROUTINE get_identifier(MOrb,NPar,tag,identifier) 
!---{{{ 
  IMPLICIT NONE
  INTEGER,INTENT(IN)            :: MOrb,NPar
  CHARACTER(len=255),INTENT(OUT):: identifier
  CHARACTER(len=55), INTENT(IN) :: tag
  CHARACTER(len=10)  :: MOrbAsString
  CHARACTER(len=1)   :: NparAsString1
  CHARACTER(len=2)   :: NparAsString2
  CHARACTER(len=3)   :: NparAsString3
  CHARACTER(len=4)   :: NparAsString4
  CHARACTER(len=5)   :: NparAsString5
  CHARACTER(len=6)   :: NparAsString6
  CHARACTER(len=7)   :: NparAsString7

IF (MOrb>0 .AND. MOrb<10) THEN
  write (MOrbAsString, '(I1)') MOrb
ELSE IF (MOrb>=10 .AND. MOrb<100) THEN
  write (MOrbAsString, '(I2)') MOrb
ELSE 
  write (*,*) "TOO MANY/FEW ORBITALS MOrb: ", MOrb
END IF

  
IF ((NPar>0) .and. (NPar<10)) THEN 
    write (NParAsString1, '(I1)') NPar
    identifier='N'//trim(NParAsString1)//'M'//trim(MOrbAsString)//trim(tag)
  ELSE  IF ((NPar>=10) .and.   (NPar<100)) THEN 
    write (NParAsString2, '(I2)') NPar
    identifier='N'//trim(NParAsString2)//'M'//trim(MOrbAsString)//trim(tag)
  ELSE IF ((NPar>=100) .and.   (NPar<1000)) THEN 
    write (NParAsString3, '(I3)') NPar
   identifier='N'//trim(NParAsString3)//'M'//trim(MOrbAsString)//trim(tag)
  ELSE IF ((NPar>=1000) .and.  (NPar<10000)) THEN 
    write (NParAsString4, '(I4)') NPar
   identifier='N'//trim(NParAsString4)//'M'//trim(MOrbAsString)//trim(tag)
  ELSE IF ((NPar>=10000) .and.  (NPar<100000)) THEN 
    write (NParAsString5, '(I5)') NPar
   identifier='N'//trim(NParAsString5)//'M'//trim(MOrbAsString)//trim(tag)
  ELSE IF ((NPar>=100000) .and. (NPar<1000000)) THEN 
    write (NParAsString6, '(I6)') NPar
    identifier='N'//trim(NParAsString6)//'M'//trim(MOrbAsString)//trim(tag)
  ELSE IF ((NPar>=1000000) .and. (NPar<10000000)) THEN 
    write (NParAsString7, '(I7)') NPar
    identifier='N'//trim(NParAsString7)//'M'//trim(MOrbAsString)//trim(tag)
  ELSE 
    write(*,*)"Npar is too big in get_identifier: ",Npar
    STOP
END IF
!---}}}
END SUBROUTINE get_identifier

!----------------------------------------------------------------------------------------
!> This routine computes the integral over the density from -infty up to x
!> Int{G1(x',x'),(x,-infty,x)} if Psi is provided.
!>
!> \authors Kaspar Sakmann (2008)
!> \since 2008
!> \todo make all use associatied variables input variables, 2d, 3d
SUBROUTINE get_cumulativeProbability(Psi,rho_jk,x,cumulativeProbability) 
USE moduleinputvariables,ONLY:NDX,NDY,NDZ,MOrb,NPar
USE moduleallocatables,ONLY:gridx
! {{{

  IMPLICIT NONE
  REAL*8,INTENT(OUT)     :: cumulativeProbability
  REAL*8,INTENT(IN)      :: x
  COMPLEX*16,DIMENSION(NDX*NDY*NDZ,MOrb),INTENT(IN) :: Psi 
  COMPLEX*16,DIMENSION(MOrb,MOrb),INTENT(IN) :: rho_jk
  INTEGER            :: n,Iorb,Jorb
  COMPLEX*16  :: G1n_n

  IF((NDY.ne.1).or.(NDZ.ne.1)) THEN
     write(*,*)"ONLY 1d implemented in get_cumulativeProbability"
     STOP
  END IF

  cumulativeProbability=0.d0

  DO n=1,NDX*NDY*NDZ
     G1n_n = (0.d0,0.d0)
     DO Jorb = 1,MOrb  
        DO Iorb=1,MOrb

           G1n_n = G1n_n + rho_jk(Iorb,Jorb) * DCONJG(Psi(n,Iorb))*Psi(n,Jorb)

        END DO
     END DO

     IF (gridx(n)<x) THEN
        cumulativeProbability = cumulativeProbability + DBLE(G1n_n)
     END IF   

  END DO
 
  cumulativeProbability = cumulativeProbability / Npar

!---}}}
END SUBROUTINE get_cumulativeProbability

!----------------------------------------------------------------------------
!> This routine writes the integral of the probability density from -infty
!> up to zero to a file
!> 
!> \authors Kaspar Sakmann (2008)
!> \since (2008)
!> \todo this routine was written for 1D only. Generalize it.
SUBROUTINE write_probabilityLeft(AbsTime,cumulativeProbability,MOrb,NPar,identifier)
!---{{{ 
IMPLICIT NONE
REAL*8,INTENT(IN)               :: AbsTime,cumulativeProbability
INTEGER, INTENT(IN)             :: MOrb,NPar
CHARACTER(255),INTENT(IN)       :: identifier
CHARACTER(255)                  :: filename
INTEGER,save                    :: counter=0

filename = 'probabilityLeft'//trim(identifier)//'.dat'

IF (counter == 0 ) THEN 

  open(unit=1979,file=filename,form='FORMATTED',status='REPLACE')
    write(1979,*)"#  "
    write(1979,*)"# the integral over the probability density in the left half of space "
    write(1979,*)"# (accurate up to one grid point)"
    write(1979,*)"#  "
    write(1979,*)"# N = trace(rho_jk) = ",NPar
    write(1979,*)"# MOrb = ",MOrb
    write(1979,*)"#  "
    write(1979,*)"# AbsTime  int_-infty^x rho^(1)(x,x) dx"
  close(1979)

END IF 

open(unit=76,file=filename,form='FORMATTED',position = 'APPEND',status   = 'OLD')
  write(76,'(2(F26.16,a1))')AbsTime," ",cumulativeProbability
close(unit=76)


counter =  1
!---}}}
END SUBROUTINE write_probabilityLeft

!----------------------------------------------------------------------------------------
!> This routine computes the total energy of the system E = <Psi|H|Psi>
!> 
!> \authors Kaspar Sakmann (2008)
!> \since 2008
!> \todo make nSuperDiags an inputvariable and get rid of the use association
SUBROUTINE get_energy(CVec,BandMat,energy,nConf)
USE   moduledefaults,ONLY:nSuperDiags
!---{{{
IMPLICIT NONE
INTEGER,INTENT(IN)      :: nConf
COMPLEX*16,INTENT(IN)   :: CVec(nConf),BandMat(nSuperDiags+1,nConf)
REAL*8,INTENT(OUT)      :: energy
COMPLEX*16              :: HCVec(nConf),cenergy

COMPLEX*16,parameter    :: alpha=(1.d0,0.d0)
COMPLEX*16,parameter    :: beta=(0.d0,0.d0)
INTEGER,parameter       :: incx=1
INTEGER,parameter       :: incy=1

HCVec=(0.d0,0.d0)

CALL ZHBMV ( 'L', nConf, nSuperDiags, alpha, BandMat, nSuperDiags+1, CVec, incx, beta, HCVec, incy )
!CALL vvaxzz (CVec,HCVec,cenergy,nConf)
cenergy = DOT_PRODUCT(Cvec,HCVec)

energy=DBLE(cenergy)

!!---}}}
END SUBROUTINE get_energy


!---------------------------------------------------------------------------
!> This routine writes the  energy <Psi|H|Psi> to a file. 
!> On the first call a header in the file is created
!>
!> \authors Kaspar Sakmann (2008)
!> \since (2008)
SUBROUTINE write_energy(AbsTime,lambda0,NPar,energy,identifier)
!---{{{
IMPLICIT NONE
REAL*8,INTENT(IN)               :: AbsTime,lambda0,energy
INTEGER,INTENT(IN)              :: NPar
INTEGER,save                    :: counter=0
CHARACTER(255),INTENT(IN)       :: identifier
CHARACTER(255)                  :: filename

filename = 'energy'//trim(identifier)//'.dat'

IF (counter == 0 ) THEN 

  open(unit=1979,file=filename,form='FORMATTED',status='REPLACE')
    write(1979,*)"#"
    write(1979,*)"# the energy of the system as a function of time"
    write(1979,*)"#"
    write(1979,*)"# N                       = ",NPar
    write(1979,*)"# lambda0                 = ",lambda0
    write(1979,*)"# lambda  = lambda0*(N-1) = ",lambda0*(NPar-1.d0)
    write(1979,*)"#"
    write(1979,*)"#    AbsTime                      energy = E = <Psi|H|Psi>   E/N "
  close(1979)


END IF 

open(unit=76,file=filename,          &
  form     = 'FORMATTED',            &
  position = 'APPEND',               &
  status   = 'OLD')
  write(76,'(3(F26.16,a1))')AbsTime," ",energy," ",energy/DBLE(NPar)
close(unit=76)

counter = 1

!---}}}
END SUBROUTINE write_energy



!---------------------------------------------------------------------------
!> This routine writes the chemical potentials mu_kj  to a file. 
!> On the first call a header in the file is created
!>
!> \authors Kaspar Sakmann (2012)
!> \since (2012)
SUBROUTINE write_mu_kj(AbsTime,mu_kj,identifier)
USE moduleinputvariables,ONLY:MOrb
!---{{{
IMPLICIT NONE
REAL*8,INTENT(IN)               :: AbsTime
COMPLEX*16,INTENT(IN)           :: mu_kj(MOrb,MOrb)
CHARACTER(255),INTENT(IN)       :: identifier
INTEGER,save                    :: counter=0
CHARACTER(255)                  :: filename
INTEGER                         :: m,n 

filename = 'mu_kj'//trim(identifier)//'.dat'

IF (counter == 0 ) THEN 

  open(unit=1979,file=filename,form='FORMATTED',status='REPLACE')
    write(1979,*)"#"
    write(1979,*)"# the chemical potentials mu_kj as a function of time"
    write(1979,*)"#"
    write(1979,*)"#"
    write(1979,*)"#    AbsTime                      mu_kj "
  close(1979)


END IF 

open(unit=76,file=filename,          &
  form     = 'FORMATTED',            &
  position = 'APPEND',               &
  status   = 'OLD')
  write(76,'(9(F26.16,a1))')AbsTime," ",( (DBLE(mu_kj(m,n))," ",IMAG(mu_kj(m,n))," ", m=1,MOrb),n=1,MOrb)
close(unit=76)

counter = 1

!---}}}
END SUBROUTINE write_mu_kj




!----------------------------------------------------------------------------------
!> This routine computes the norm ||d|| of the defect d in the equation
!> H|Psi> = E|Psi> + d. Only the coefficent vector is used. 
!> The routine is useful in relaxation computations to check
!> whether an eigenstate is reached. In that case ||d||=0 is satisfied.
!>
!> \authors Kaspar Sakmann (2011)
!> \since 2011
SUBROUTINE get_defect(nConf,nSuperDiags,BandMat,CVec,energy,defect)
!---{{{ 
IMPLICIT NONE
INTEGER,INTENT(IN)              :: nConf,nSuperDiags
COMPLEX*16,INTENT(IN)           :: CVec(nConf),BandMat(nSuperDiags+1,nConf)
REAL*8,INTENT(IN)               :: energy

REAL*8,INTENT(OUT)              :: defect

COMPLEX*16                      :: cdefect=(0.d0,0.d0)
COMPLEX*16                      :: alpha,beta
COMPLEX*16                      :: HCVec(nConf)
INTEGER                         :: incx,incy

incx=1
incy=1
alpha=(1.d0,0.d0)
beta =(0.d0,0.d0)

HCVec=(0.d0,0.d0)

CALL ZHBMV ( 'L', nConf, nSuperDiags, alpha, BandMat, nSuperDiags+1, CVec, incx, beta,HCVec, incy)

cdefect = DOT_PRODUCT(HCVec,HCVec)

!get d = H|Psi> - E|Psi>
HCVec = HCVec - energy * CVec

cdefect = DOT_PRODUCT(HCVec,HCVec)
defect  = SQRT(ABS(cdefect))

!---}}}
END SUBROUTINE get_defect


!----------------------------------------------------------------------------------
!> This routine computes the mean density of the system Int[rho(x)**2 / N].
!> It is used to compute the Lieb Liniger parameter for inhomogeneous 1D systems, 
!> as defined in Lieb, Seiringer PRL 91, 105401 (2003).
!>
!> \authors Kaspar Sakmann (2009)
!> \since 2009
!> \todo get rid of the use association, make weight an input parameter, 2d,3d
SUBROUTINE get_meanDensity(Psi,rho_jk,meanDensity) 
USE moduleinputvariables
! computes Int{G1(x',x')^2,(x,-infty,x)} if Psi is provided {{{

  IMPLICIT NONE
  REAL*8,INTENT(OUT)     :: meanDensity
  REAL*8                 :: w
  COMPLEX*16,DIMENSION(NDX*NDY*NDZ,MOrb),INTENT(IN) :: Psi 
  COMPLEX*16,DIMENSION(MOrb,MOrb),INTENT(IN) :: rho_jk
  INTEGER            :: n,Iorb,Jorb
  COMPLEX*16  :: cdensxsquared


  IF((NDY.ne.1).or.(NDZ.ne.1)) THEN
     write(*,*)"ONLY 1d implemented"
     STOP
  END IF
  
  w=DSQRT((xf-xi)/NDX)
  meanDensity=0.d0

  DO n=1,NDX*NDY*NDZ
     cdensxsquared = (0.d0,0.d0)
     DO Jorb = 1,MOrb  
        DO Iorb=1,MOrb

           cdensxsquared = cdensxsquared + rho_jk(Iorb,Jorb) * DCONJG(Psi(n,Iorb))*Psi(n,Jorb)

        END DO
     END DO
     meanDensity = meanDensity + DBLE(cdensxsquared)**2/w**2

  END DO
 
  meanDensity = meanDensity / Npar

!---}}}
END SUBROUTINE get_meanDensity

!---------------------------------------------------------------------------
!> This routine writes the generalized Lieb-Liniger parameter to a file. 
!> On the first call a header in the file is created
!>
!> \authors Kaspar Sakmann (2008)
!> \since (2008)
SUBROUTINE write_liebLinigerParameter(AbsTime,lambda0,NPar,meanDensity,identifier)
!---{{{
IMPLICIT NONE
REAL*8,INTENT(IN)               :: AbsTime,lambda0,meanDensity
INTEGER,INTENT(IN)              :: NPar
INTEGER,save                    :: counter=0
CHARACTER(255),INTENT(IN)       :: identifier
CHARACTER(255)                  :: filename

filename = 'liebLinigerParameter'//trim(identifier)//'.dat'


IF (counter == 0 ) THEN 

  open(unit=1979,file=filename,form='FORMATTED',status='REPLACE')
    write(1979,*)"#"
    write(1979,*)"# the Lieb Liniger parameter as a function of time"
    write(1979,*)"# cf The mathematics of the Bose-gas, Lieb, Seiringer, Solovej, Yngvason"
    write(1979,*)"# we use the definition that reduces to gamma of LL's 1963 paper here "
    write(1979,*)"# "
    write(1979,*)"# AbsTime  gamma=lambda0/(int [rho^(1)(x,x)]^2 / N dx)"
    write(1979,*)"# N                       = ",NPar
    write(1979,*)"# lambda0                 = ",lambda0
    write(1979,*)"# lambda  = lambda0*(N-1) = ",lambda0*(NPar-1.d0)
  close(1979)


END IF 

open(unit=76,file=filename,          &
  form     = 'FORMATTED',            &
  position = 'APPEND',               &
  status   = 'OLD')
  write(76,'(2(F26.16,a1))')AbsTime," ",lambda0/meanDensity
close(unit=76)

counter = 1

!---}}}
END SUBROUTINE write_liebLinigerParameter

!----------------------------------------------------------------------------------
!> This routine returns the natural orbitals (norbs) and their occupations (noccs) given 
!> the reduced one-body density matrix rho_jk and the corresponding orbitals array Psi
!>
!> \authors Kaspar Sakmann (2008)
!> \since 2008
SUBROUTINE get_naturalOrbitals(MOrb,NDX,NDY,NDZ,rho_jk,Psi,noccs,norbs)
USE moduleparameters
!{{{

IMPLICIT NONE
INTEGER,INTENT(IN)      :: MOrb,NDX,NDY,NDZ
COMPLEX*16,INTENT(IN)   :: rho_jk(MOrb,MOrb)
COMPLEX*16,INTENT(IN)   :: Psi(NDX*NDY*NDZ,MOrb)
REAL*8 ,INTENT(OUT)     :: noccs(MOrb)
COMPLEX*16,INTENT(OUT)  :: norbs(NDX*NDY*NDZ,MOrb)

COMPLEX*16        :: A(MOrb,MOrb),WORK(2*MOrb)
REAL*8            :: RWORK(max(1, 3*MOrb-2))
INTEGER           :: INFO
INTEGER           :: m,n

RWORK = 0.d0
INFO  = 0

! to get the noccs in descENDing order muliply rho_jk by -1

A     = (-1.d0) * rho_jk  
CALL ZHEEV('V','U',MOrb,A,MOrb,noccs,WORK,2*MOrb,RWORK,INFO)
 IF(INFO.NE.0) THEN
  WRITE(*,*)"THE SHIT HAS HIT THE FAN IN get_naturalOrbitals"
  WRITE(*,*)"INFO",INFO
  STOP
 END IF

noccs = noccs * (-1.d0)
! on output the columns of A are the eigenvecors of rho_jk
! the eigenvalUSE are stored in noccs 
! with U = A and lambda = noccs (diagonal matrix) 
! rho = U lambda U^dagger 
! the m-th norb is THEN norb(:,m) = Sum_j  Psi(:,j) A^*_jm

norbs = ZERO

DO m=1,MOrb
  DO n=1,MOrb

    norbs(:,m) = norbs(:,m) + Psi(:,n)*DCONJG(A(n,m))

  END DO
END DO

!}}}
END SUBROUTINE get_naturalOrbitals

!---------------------------------------------------------------------------------
!> This routine writes the orbital array to a file. If fileID == 0, the file   
!> <AbsTime>.orbs.dat is written, else if fileID == 1 the file restart.orbs.out
!> is written
!>
!> \authors Kaspar Sakmann (2007)
!> \since (2007)
SUBROUTINE write_PsiToFile(AbsTime,Psi,NDX,NDY,NDZ,MOrb,fileID,identifier)
USE moduleallocatables,ONLY:gridx,gridy,gridz,V_ext
!       {{{ 
IMPLICIT NONE
REAL*8,INTENT(IN)               :: AbsTime
COMPLEX*16,INTENT(IN)           :: Psi(NDX*NDY*NDZ,MOrb)
INTEGER,INTENT(IN)              :: fileID,NDX,NDY,NDZ,MOrb

CHARACTER (LEN=10)              :: doubleAsString
INTEGER                         :: outputunit
INTEGER                         :: i,j,k,m,n

CHARACTER(255),INTENT(IN)       :: identifier
CHARACTER(255)                  :: filename

outputunit        =       11
CALL get_doubleAsString(AbsTime,doubleAsString)


IF (fileID==0) THEN

   filename=trim(doubleAsString//trim(identifier)//'orbs.dat')

ELSE 

   filename=trim('restart.orbs.out')

END IF


open(unit=outputunit,file=trim(filename),form='formatted') 
    DO k=1,NDZ ! 
      DO j=1,NDY ! 
        DO i=1,NDX ! format: x y z V_ext  Re(Psi_1) Im(Psi_1),...,Re(Psi_M) Im(Psi_M)

        write(11,*)gridx(i)," ",gridy(j)," ",gridz(k)," ",DBLE(V_ext(i+NDX*(j-1)+NDX*NDY*(k-1)))," ", & 
                  ( DBLE(Psi(i+NDX*(j-1)+NDX*NDY*(k-1),m))," ",&
                    DIMAG(Psi(i+NDX*(j-1)+NDX*NDY*(k-1),m)), m=1,MOrb)

        END DO
      END DO
    END DO

close(outputunit)

!       }}}
END SUBROUTINE write_PsiToFile 

!---------------------------------------------------------------------------
!> This routine writes the natural orbital array to a file
!>
!> \authors Kaspar Sakmann (2008)
!> \since (2008)
!> \todo: get rid of the use association
SUBROUTINE write_norbsToFile(AbsTime,norbs,NDX,NDY,NDZ,MOrb,identifier)
USE moduleallocatables,ONLY:gridx,gridy,gridz,V_ext
!       {{{ write an output file. all vectors contain the weights
IMPLICIT NONE
REAL*8,INTENT(IN)   :: AbsTime
COMPLEX*16,INTENT(IN)     :: norbs(NDX*NDY*NDZ,MOrb)
INTEGER,INTENT(IN)            :: NDX,NDY,NDZ,MOrb
CHARACTER(255),INTENT(IN)     :: identifier
CHARACTER (LEN=10)            :: doubleAsString
CHARACTER(255)                :: filename

INTEGER                       :: outputunit
INTEGER                       :: i,j,k,m,n

outputunit        =       11
CALL get_doubleAsString(AbsTime,doubleAsString)

filename=trim(doubleAsString//trim(identifier)//'naturalorbitals.dat')

open(unit=outputunit,file=trim(filename),form='formatted') 
    DO k=1,NDZ ! 
      DO j=1,NDY ! 
        DO i=1,NDX ! format: x y z V_ext  Re(norbs_1) Im(norbs_1),...,Re(norbs_M) Im(norbs_M)

        write(outputunit,*)gridx(i)," ",gridy(j)," ",gridz(k)," ",DBLE(V_ext(i+NDX*(j-1)+NDX*NDY*(k-1)))," ", & 
                  ( DBLE(norbs(i+NDX*(j-1)+NDX*NDY*(k-1),m))," ",&
                    DIMAG(norbs(i+NDX*(j-1)+NDX*NDY*(k-1),m)), m=1,MOrb)

        END DO
      END DO
    END DO

close(outputunit)

!       }}}
END SUBROUTINE write_norbsToFile 

!---------------------------------------------------------------------------
!> This routine writes the coefficent vector CVec to a file. 
!> If fileID == 0, the file <AbsTime>.coef.dat is written
!> else if fileID == 1, the file restart.coef.out
!>
!> \authors Kaspar Sakmann (2008)
!> \since (2008)
!> \todo: get rid of the use association
SUBROUTINE write_CVecFile(AbsTime,CVec,fileID,identifier)
USE moduledefaults,ONLY:nConf
!       {{{ 
IMPLICIT NONE
REAL*8,INTENT(IN)                       :: AbsTime
COMPLEX*16,INTENT(IN)                   :: CVec(nConf)
INTEGER,INTENT(IN)                      :: fileID
CHARACTER (LEN=255),INTENT(IN)          :: identifier

CHARACTER (LEN=10)        :: doubleAsString
CHARACTER (LEN=50)        :: filename

INTEGER                   :: outputunit
INTEGER                   :: i

outputunit        =       11

CALL get_doubleAsString(AbsTime,doubleAsString)

IF (fileID==0) THEN

   filename=trim(doubleAsString//trim(identifier)//'coef.dat')

ELSE 

   filename=trim('restart.coef.out')

END IF

open(unit=outputunit,file=trim(filename),form='formatted') 

    DO i=1,nConf ! format: i    Re(CVec(i))    Im(CVec(i))   
        IF (ABS(CVec(i))>0.5d-14) THEN
          write(11,912)i," ",dreal(CVec(i))," ",dimag(CVec(i))
        END IF
    END DO

close(outputunit)

912   format(I12,a3,f18.15,a3,f18.15)
!       }}}
END SUBROUTINE write_CVecFile 

!---------------------------------------------------------------------------
!> This routine reads the external potential V_ext from the file 
!> V_ext.inp. Only the potential (fourth column) is read in, nothing else.
!> All other columns are ignored. This allows to read in any potential
!> from an orbital file of a previous run. 
!>
!> \authors Kaspar Sakmann (2007)
!> \since (2007)
!> \todo: make the name of the file an option in input.nml
SUBROUTINE read_potentialFromFile(V_ext,NDX,NDY,NDZ)
! {{{ 
IMPLICIT NONE
INTEGER,INTENT(IN)        :: NDX,NDY,NDZ
REAL*8,INTENT(OUT)        :: V_ext(NDX*NDY*NDZ)
INTEGER                   :: inputunit,ierr
INTEGER                   :: i,j,k,m,nvals
REAL*8                    :: x,y,z

inputunit       = 11
ierr            = 0
x               = 0.d0
y               = 0.d0
z               = 0.d0
V_ext           = 0.d0
nvals           = 0

WRITE(*,100)"--> Reading external potential from file V_ext.inp"

OPEN(UNIT=inputunit,FILE='V_ext.inp',FORM='formatted',ACTION='READ',IOSTAT=ierr) 

openif: IF (ierr == 0) THEN 

    readloop: DO k=1,NDZ  
      DO j=1,NDY  
        DO i=1,NDX ! format: x,y,z, V(x,y,z)   
        
          READ(inputunit,*,IOSTAT=ierr)x,y,z,V_ext(i+NDX*(j-1)+NDX*NDY*(k-1)) 
          IF(ierr /= 0) EXIT
          nvals = nvals + 1

        END DO
      END DO
    END DO readloop

    readif: IF (ierr > 0) THEN
      WRITE(*,1020) nvals+1
      1020 FORMAT ('0','AN ERROR OCCURED READING LINE:',I6)
      STOP
    ELSE IF (ierr < 0) THEN
      WRITE(*,1030) nvals,NDX*NDY*NDZ
      1030 FORMAT(' ','TRIED READING BEYOND THE LAST LINE:',I6,', EXPECTED',I6,' LINES')
      STOP
    ELSE 
        CONTINUE
    END IF readif


ELSE openif
    WRITE(*,1040) ierr
    1040 FORMAT (' ','Error opening file: IOSTAT = ', I6)
END IF openif

CLOSE(inputunit)

!WRITE(*,100)"<-- Reading external potential successful"

100 FORMAT(5X,A)
!       }}}
END SUBROUTINE read_potentialFromFile 

!---------------------------------------------------------------------------
!> This routine reads the array of orbitals Psi from the file 
!> restart.orbs.inp. The orbitals are orthonormalized.
!>
!> \authors Kaspar Sakmann (2007)
!> \since (2007)
SUBROUTINE read_PsiFromFile(Psi,NDX,NDY,NDZ,MOrb)
USE moduleparameters
!       {{{ read the orbitals from a file and orthonormalize them 

IMPLICIT NONE
INTEGER,INTENT(IN)        :: NDX,NDY,NDZ,MOrb
COMPLEX*16,INTENT(OUT)    :: Psi(NDX*NDY*NDZ,MOrb)
INTEGER                   :: inputunit,ierr
INTEGER                   :: i,j,k,m,n,nvals
REAL*8                    :: x,y,z,V_xyz,RePsi(MOrb),ImPsi(MOrb),norm

inputunit       = 11
ierr            = 0
x               = 0.d0
y               = 0.d0
z               = 0.d0
V_xyz           = 0.d0
RePsi           = 0.d0
ImPsi           = 0.d0
nvals           = 0
norm            = 0.d0

WRITE(*,100)"--> Reading orbitals from restart.orbs.inp"

OPEN(UNIT=inputunit,FILE='restart.orbs.inp',FORM='formatted',ACTION='READ',IOSTAT=ierr) 

openif: IF (ierr == 0) THEN 

    readloop: DO k=1,NDZ ! 
      DO j=1,NDY ! 
        DO i=1,NDX ! format: x,y,z, V(x,y,z)   Re(Psi_1)    Im(Psi_1),...,Re(Psi_MOrb)    Im(Psi_MOrb) 
        
          READ(inputunit,*,IOSTAT=ierr)x,y,z,V_xyz,(RePsi(m), ImPsi(m),m=1,MOrb) 
          
          IF(ierr /= 0) THEN
            WRITE(*,*) x,y,z,V_xyz
            EXIT
          END IF    
          nvals = nvals + 1

          DO m=1,MOrb
            Psi(i+NDX*(j-1)+NDX*NDY*(k-1),m) = RePsi(m) + ZONEI * ImPsi(m)  
          END DO
   
        END DO
      END DO
    END DO readloop

    readif: IF (ierr > 0) THEN
      WRITE(*,1020) nvals+1
      1020 FORMAT ('0',' IN read_PsiFromFile: AN ERROR OCCURED READING LINE',I6)
      STOP
    ELSE IF (ierr < 0) THEN
      WRITE(*,1030) nvals,NDX*NDY*NDZ
      1030 FORMAT(' ',' IN read_PsiFromFile: TRIED READING BEYOND THE LAST LINE OF FILE.',I6, &
                  ' LINES READ IN, BUT EXPECTED',I6)
      STOP
    ELSE 
      CONTINUE
    END IF readif


ELSE openif
    WRITE(*,1040) ierr
    1040 FORMAT (' ','Error opening file: IOSTAT = ', I6)
END IF openif
CLOSE(inputunit)

DO m=1,MOrb
  CALL normvxz(Psi(:,m),norm,NDX*NDY*NDZ)
  WRITE(*,105)"orbital",m," has norm ",norm
END DO


WRITE(*,110)"Gram-Schmidt orthonormalization of the orbitals"
CALL schmidtortho (Psi,NDX*NDY*NDZ,MOrb,ierr)
IF (Ierr.NE.0) THEN
   WRITE(*,*)"orthogonalization failed"
END IF

!WRITE(*,100)"<-- Reading orbitals succesfull"
100 FORMAT (5X,A)
105 FORMAT (9X,A,I3,A,F16.8)
110 FORMAT (9X,A)

!       }}}
END SUBROUTINE read_PsiFromFile 

!---------------------------------------------------------------------------
!> This routine reads the array of coefficients from the file 
!> restart.coef.inp. The vector is normalized in the process.
!>
!> \authors Kaspar Sakmann (2008)
!> \since (2008)
SUBROUTINE read_CVecFile(CVec,nConf)
!       {{{ 
IMPLICIT NONE
INTEGER,INTENT(IN)           :: nConf
COMPLEX*16,INTENT(OUT)   :: CVec(nConf)
REAL*8                       :: RE_CVec(nConf),IM_CVec(nConf),norm
REAL*8                       :: RE,IM

INTEGER                      :: inputunit
INTEGER                      :: i,j

WRITE(*,90)"--> Reading the coefficient vector"

CVec    = (0.d0,0.d0)
RE_CVec = 0.0d0
IM_CVec = 0.0d0
IM      = 0.0D0
RE      = 0.0D0
inputunit        =       11
norm    = 0.d0


open(unit=inputunit,file='restart.coef.inp',form='formatted') 

  DO i=1,nConf ! format: i    Re(CVec(i))    Im(CVec(i))   
    read(inputunit,*,END=21)j,RE,IM
    IF (j<=nConf) THEN 
      RE_CVec(j)=RE
      IM_CVec(j)=IM
    ELSE
      WRITE(*,*)"THE VECTOR CVec HAS ONLY nConf CONFIGURATIONS, nConf:",nConf
      WRITE(*,*)"ERROR, I TRIED TO READ CONFIGURATION NO ",j
      STOP
    END IF
  END DO
21  continue

close(inputunit)


CVec = RE_CVec + (0.d0,1.d0)*IM_CVec
CALL normvxz(CVec,norm,nConf)
WRITE(*,100)"    Norm of read in coefficient vector =",norm
WRITE(*,100)"    Normalize the coefficient vector"

CVec = CVec/norm
CALL normvxz(CVec,norm,nConf)

90  FORMAT(5X,A)
100 FORMAT(5X,A,F16.12)

!WRITE(*,90)"<-- Reading coefficient vector successful"
!       }}}
END SUBROUTINE read_CVecFile 

!---------------------------------------------------------------------------
!> This routine writes the natural orbital occupations noccs to a file. 
!> On the first call a header in the file is created
!>
!> \authors Kaspar Sakmann (2007)
!> \since (2007)
SUBROUTINE write_noccsToFile(AbsTime,MOrb,noccs,identifier)
!---{{{
  IMPLICIT NONE
  INTEGER, INTENT(IN)           :: MOrb
  REAL*8, INTENT(IN)            :: AbsTime,noccs(MOrb)
  CHARACTER(len=255),INTENT(IN) :: identifier      

  INTEGER,save          :: counter=0
  CHARACTER (len=255)   :: fullName,varfmt,MOrbString
  CHARACTER (len=6)     :: posString
  REAL*8                :: trace
  INTEGER               :: m


  trace = 0.d0

  DO m=1,MOrb
    trace = trace + noccs(m)
  END DO

  fullName          =      'naturalOccupations'//trim(identifier)//'.dat'

  IF (MOrb<9 .AND. MOrb>0) THEN
    write(MOrbString,'(I1)') MOrb+1
    varfmt = "("//trim(MOrbString)//"(F20.10,a1))"
  ELSE IF (MOrb<99 .AND. MOrb > 9) THEN
    write(MOrbString,'(I2)') MOrb+1
    varfmt = "("//trim(MOrbString)//"(F20.10,a1))"
  ELSE
    WRITE(*,*)"TOO MANY/FEW ORBITALS: MOrb",MOrb
    STOP
  END IF

  IF(counter==0) THEN 
    open(unit=12,file=trim(fullName),form='formatted',POSITION='REWIND')
    write(12,*)"# (fractional) natural orbital occupations"
    write(12,*)"# "
    write(12,*)"# N = trace(rho_jk) = ",trace
    write(12,*)"# MOrb = ",MOrb
    write(12,*)"# "
    write(12,*)"#      time                 ",( "n_m/N                ", m=1,MOrb)
    close(12)
  END IF 

  open(unit=12,file=fullName,form='formatted',POSITION='APPEND')
    write(12,FMT=trim(varfmt)) AbsTime," ",(noccs(m)/trace, " ", m=1,MOrb)
  close(12)

  counter=1
!}}}
END SUBROUTINE write_noccsToFile

!---------------------------------------------------------------------------
!> This routine writes the times at which a certain fragmentation is reached
!> for the first time to a file. 
!> On the first call a header in the file is created
!>
!> \authors Kaspar Sakmann (2010)
!> \since (2010)
SUBROUTINE write_firstFragmentationTimes(AbsTime,MOrb,NPar,noccs,lambda0,identifier)
!---{{{
IMPLICIT NONE
INTEGER, INTENT(IN)             :: MOrb,NPar
REAL*8, INTENT(IN)              :: AbsTime,noccs(MOrb),lambda0
CHARACTER (len=255),INTENT(IN)  :: identifier
 
INTEGER,save                    :: counter=0,fragcounter=0
CHARACTER (len=255)             :: fullName
CHARACTER (len=6)               :: posString
REAL*8                          :: frag,increment,trace,lambda
INTEGER                         :: m

trace = 0.d0
DO m =1,MOrb  
  trace = trace + noccs(m)
END DO

lambda = (NPar-1.d0)*lambda0

  increment=0.002d0
 
  fullName          =      'firstFragmentationTimes'//trim(identifier)//'.dat'
 
  frag = 1.d0 - noccs(1)/trace ! the fragmentation in percent 1.0 - n^(1)_1 / N 
 
  posString='APPEND'
  IF(counter==0) THEN 
  posString='REWIND'      

  open(unit=12,file=trim(fullName),form='formatted',POSITION=posString)
  write(12,*)"# times at which a certain fragmetation is reached for the first time"
  write(12,*)"# "
  write(12,*)"# trace(rho_jk) = ",trace
  write(12,*)"# NPar          = ",NPar
  write(12,*)"# "
  write(12,*)"#     fragmentation          tbreak                 N                      lambda=lambda0*(N-1)"
  write(12,222) frag," ",AbsTime," ",DBLE(NPar)," ",lambda
  close(12)
  posString='APPEND'
  END IF 

  IF (frag>increment*(fragcounter+1)) THEN

    while_loop: DO 

      fragcounter=fragcounter+1
      open(unit=12,file=fullName,form='formatted',POSITION=posString)
        write(12,222) fragcounter*increment," ",AbsTime," ",DBLE(NPar)," ",lambda
      close(12)

      IF (frag<(fragcounter +1)*increment) exit while_loop

    END DO while_loop 

  END IF



222 format(4(F22.13,a1))
  counter=1
!}}}
END SUBROUTINE write_firstFragmentationTimes

END MODULE moduleinputoutput
