! vim:fdm=marker:

!---------------------------------------------------------------------------
!> This module contains the declarations for the variables read in via input.nml
!> and the associated NAMELISTS.
!> IMPORTANT:
!> the default values defined here will only be used, if the respective 
!> NAMELISTS are NOT read in via 'input.nml'
!>
!>\authors Kaspar Sakmann (2007)
!>\since  2007
!>\authors Axel Lode (2011)
!>\since  2011

MODULE moduleinputvariables
IMPLICIT NONE
SAVE

!SystemParameters
REAL*8  :: lambda0             =       0.0000D0
INTEGER :: NPar                =       100
INTEGER :: MOrb                =       2
INTEGER :: Dims                =       1
LOGICAL :: useIMEST            =       .FALSE.
NAMELIST /SystemParameters/ lambda0 , NPar, MOrb, Dims, useIMEST

!IMESTParameters
REAL*8  :: imestParameter1     =        1.d0
NAMELIST /IMESTParameters/ imestParameter1

!GridParameters
INTEGER :: NDX                  =       251
INTEGER :: NDY                  =       1
INTEGER :: NDZ                  =       1
REAL*8  :: xi                   =      -10.d0
REAL*8  :: xf                   =       10.d0
REAL*8  :: yi                   =        0.d0
REAL*8  :: yf                   =        0.d0
REAL*8  :: zi                   =        0.d0
REAL*8  :: zf                   =        0.d0
NAMELIST /GridParameters/ xi,xf,yi,yf,zi,zf,NDX,NDY,NDZ

!PrintParameters
REAL*8  :: tinyStep             =       0.1d0
REAL*8  :: printStep            =       5.d0
REAL*8  :: printAllStep         =       100.d0
INTEGER :: printCorrelationFuncs      =       0
!PrintParameters for 2D analysis routines.
LOGICAL :: REALSPACE2D = .FALSE.
LOGICAL :: MOMSPACE2D = .FALSE.
REAL*8  :: x1slice    = 0.d0
REAL*8  :: x2slice    = 0.d0
REAL*8  :: y1slice    = 0.d0
REAL*8  :: y2slice    = 0.d0
LOGICAL :: x1const    = .TRUE.
LOGICAL :: y1const    = .TRUE.
LOGICAL :: x2const    = .FALSE.
LOGICAL :: y2const    = .FALSE.
LOGICAL :: Pnot       = .FALSE.
REAL*8  :: xstart     = -10.d0
REAL*8  :: xstop     = -10.d0


NAMELIST /PrintParameters/ tinyStep, printStep, printAllStep, printCorrelationFuncs,&
                           MOMSPACE2D,REALSPACE2D,&
                           x1const,x1slice,&
                           y1const,y1slice,&
                           x2const,x2slice,&
                           y2const,y2slice,&
                           Pnot,xstart,xstop
                             
!RunParameters
REAL*8  :: AbsTime              =       0.0d0
REAL*8  :: maxTime              =       100.d0
REAL*8  :: TolError             =       1.d-06
LOGICAL :: restart              =       .TRUE.
LOGICAL :: Relax                =       .FALSE. 
LOGICAL :: readPotential        =       .FALSE. 
INTEGER :: propDirection        =       1 
character(55)  :: tag           =       ''   
NAMELIST /RunParameters/ AbsTime, maxTime, TolError,restart,Relax,readPotential, propDirection,tag


END MODULE moduleinputvariables

